; Copyright 2004-2010 John T. Haller of PortableApps.com
; Copyright 2008-2010 Vic Saville

; Website: http://portableapps.com/apps/office/foxit_reader_portable

; This software is OSI Certified Open Source Software.
; OSI Certified is a certification mark of the Open Source Initiative.

; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; as published by the Free Software Foundation; either version 2
; of the License, or (at your option) any later version.

; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.

; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

!define PORTABLEAPPNAME "Foxit Reader Portable"
!define APPNAME "Foxit Reader"
!define NAME "FoxitReaderPortable"
!define VER "1.6.8.0"
!define WEBSITE "portableapps.com/apps/office/foxit_reader_portable"
!define DEFAULTEXE "Foxit Reader.exe"
!define DEFAULTAPPDIR "Foxit Reader"
!define DEFAULTSETTINGSDIR "settings"
!define LAUNCHERLANGUAGE "English"

;=== Program Details
Name "${PORTABLEAPPNAME}"
OutFile "..\..\${NAME}.exe"
Caption "${PORTABLEAPPNAME} | PortableApps.com"
VIProductVersion "${VER}"
VIAddVersionKey ProductName "${PORTABLEAPPNAME}"
VIAddVersionKey Comments "Allows ${APPNAME} to be run from a removable drive.  For additional details, visit ${WEBSITE}"
VIAddVersionKey CompanyName "PortableApps.com"
VIAddVersionKey LegalCopyright "PortableApps.com & Contributors"
VIAddVersionKey FileDescription "${PORTABLEAPPNAME}"
VIAddVersionKey FileVersion "${VER}"
VIAddVersionKey ProductVersion "${VER}"
VIAddVersionKey InternalName "${PORTABLEAPPNAME}"
VIAddVersionKey LegalTrademarks "PortableApps.com is a Trademark of Rare Ideas, LLC."
VIAddVersionKey OriginalFilename "${NAME}.exe"
; VIAddVersionKey PrivateBuild ""
; VIAddVersionKey SpecialBuild ""

;=== Runtime Switches
CRCCheck on
WindowIcon off
SilentInstall silent
AutoCloseWindow true
RequestExecutionLevel user
XPStyle on

; Best Compression
SetCompress auto
SetCompressor /SOLID lzma
SetCompressorDictSize 32
SetDatablockOptimize on

;=== Include
; (Standard NSIS)
!include FileFunc.nsh
!insertmacro GetParameters
!include LogicLib.nsh
!include Registry.nsh

; (NSIS Plugins)
!include TextReplace.nsh

; (Custom)
!addincludedir Include
!include ReadINIStrWithDefault.nsh
!include ReplaceInFileWithTextReplace.nsh

;=== Program Icon
Icon "..\..\App\AppInfo\appicon.ico"

;=== Languages
LoadLanguageFile "${NSISDIR}\Contrib\Language files\${LAUNCHERLANGUAGE}.nlf"
!include PortableApps.comLauncherLANG_${LAUNCHERLANGUAGE}.nsh

;=== Variables
Var ADDITIONALPARAMETERS
Var APPLANGUAGE
Var CURRENTDRIVE
Var DISABLESPLASHSCREEN
Var EXECSTRING
Var FAILEDTORESTOREKEY
Var INIPATH
Var LASTDRIVE
Var MISSINGFILEORPATH
Var PROGRAMDIRECTORY
Var PROGRAMEXECUTABLE
Var RegKeyFDFExists
Var RegKeyPDFExists
Var RegKeyPDFIsBlank
Var SECONDARYLAUNCH
Var SETTINGSDIRECTORY
Var USERTYPE

Section "Main"
		StrCpy $PROGRAMDIRECTORY "$EXEDIR\App\${DEFAULTAPPDIR}"
		StrCpy $PROGRAMEXECUTABLE "${DEFAULTEXE}"
		StrCpy $SETTINGSDIRECTORY "$EXEDIR\Data\${DEFAULTSETTINGSDIR}"
		StrCpy $DISABLESPLASHSCREEN "false"
		;=== Find the INI file, if there is one
		IfFileExists "$EXEDIR\${NAME}.ini" "" NoINI
			StrCpy $INIPATH "$EXEDIR"
			;=== Read the parameters from the INI file
			${ReadINIStrWithDefault} $ADDITIONALPARAMETERS "$INIPATH\${NAME}.ini" "${NAME}" "AdditionalParameters" ""
			Goto FindProgramEXE

	NoINI:
		;=== No INI file, so we'll use the defaults
		StrCpy $ADDITIONALPARAMETERS ""

	FindProgramEXE:
		IfFileExists "$PROGRAMDIRECTORY\$PROGRAMEXECUTABLE" FoundProgramEXE
			;=== Program executable not where expected
			StrCpy $MISSINGFILEORPATH $PROGRAMEXECUTABLE
			MessageBox MB_OK|MB_SETFOREGROUND|MB_ICONEXCLAMATION `$(LauncherFileNotFound)`
			Goto TheEnd

	FoundProgramEXE:
		;=== Check if launcher is already running
		System::Call 'kernel32::CreateMutexA(i 0, i 0, t "${NAME}") i .r1 ?e'
		Pop $0
		StrCmp $0 "0" CheckAppRunning
			StrCpy $SECONDARYLAUNCH "true"
			FindProcDLL::FindProc "$PROGRAMEXECUTABLE"
			StrCmp $R0 "1" DisplaySplash
			MessageBox MB_OK|MB_SETFOREGROUND|MB_ICONINFORMATION `$(LauncherClosing)`
			Goto TheEnd

	CheckAppRunning:
		FindProcDLL::FindProc "$PROGRAMEXECUTABLE"
		StrCmp $R0 "1" "" DisplaySplash

	; WarnAnotherInstance:
		MessageBox MB_OK|MB_SETFOREGROUND|MB_ICONINFORMATION `$(LauncherAlreadyRunning)`
		Goto TheEnd

	DisplaySplash:
		StrCmp $DISABLESPLASHSCREEN "true" GetPassedParameters
			;=== Show the splash screen before processing the files
			InitPluginsDir
			File /oname=$PLUGINSDIR\splash.jpg "${NAME}.jpg"
			newadvsplash::show /NOUNLOAD 0 0 0 -1 /L "$PLUGINSDIR\splash.jpg" ; splash is off

	GetPassedParameters:
		${GetParameters} $0
		StrCmp "'$0'" "''" "" LaunchProgramParameters

		;=== No parameters
		StrCpy $EXECSTRING `"$PROGRAMDIRECTORY\$PROGRAMEXECUTABLE"`
		Goto AdditionalParameters

		LaunchProgramParameters:
			StrCpy $EXECSTRING `"$PROGRAMDIRECTORY\$PROGRAMEXECUTABLE" $0`

		AdditionalParameters:
			StrCmp $ADDITIONALPARAMETERS "" GetPassedParametersEnd

			;=== Additional Parameters
			StrCpy $EXECSTRING `$EXECSTRING $ADDITIONALPARAMETERS`

	GetPassedParametersEnd:

	; SecondaryLaunch:
		StrCmp $SECONDARYLAUNCH "true" LaunchAndExit

	; GetPaths:
		${GetRoot} $EXEDIR $CURRENTDRIVE

	; CheckSettings:
		IfFileExists "$SETTINGSDIRECTORY\*.*" CheckSettingsEnd

			;=== Copy the default settings files
			CreateDirectory "$SETTINGSDIRECTORY"
			CopyFiles /SILENT "$EXEDIR\App\DefaultData\settings\*.*" "$SETTINGSDIRECTORY"

			;=== Check InitialOpenDir
			IfFileExists "$CURRENTDRIVE\Documents\*.*" 0 +3
			WriteINIStr "$SETTINGSDIRECTORY\${NAME}.reg" "HKEY_CURRENT_USER\Software\Foxit Software\Foxit Reader\ChildFrame" `"InitialOpenDir"` `"$CURRENTDRIVE\\Documents\\"`
			Goto CheckSettingsEnd
			WriteINIStr "$SETTINGSDIRECTORY\${NAME}.reg" "HKEY_CURRENT_USER\Software\Foxit Software\Foxit Reader\ChildFrame" `"InitialOpenDir"` `"$CURRENTDRIVE\\"`

	CheckSettingsEnd:

	; AdjustDrives:
		ReadINIStr $LASTDRIVE "$SETTINGSDIRECTORY\${NAME}Settings.ini" "${NAME}Settings" "LastDrive"
		StrCmp $LASTDRIVE $CURRENTDRIVE AdjustDrivesEnd
			${ReplaceInFile} '$SETTINGSDIRECTORY\${NAME}.reg' '$LASTDRIVE\\' '$CURRENTDRIVE\\'

			;=== Store current drive
			WriteINIStr "$SETTINGSDIRECTORY\${NAME}Settings.ini" "${NAME}Settings" "LastDrive" "$CURRENTDRIVE"

	AdjustDrivesEnd:

	; Language:
		IfFileExists "$PROGRAMDIRECTORY\lang_*.xml" "" LanguageEnd ; no languages installed
			ReadEnvStr $APPLANGUAGE "PortableApps.comLocaleID"
			StrCmp $APPLANGUAGE "" LanguageEnd ; if not set, move on
			StrCmp $APPLANGUAGE "1033" "" GetAppLanguage ; English is built-in
			WriteINIStr "$SETTINGSDIRECTORY\${NAME}.reg" "HKEY_CURRENT_USER\Software\Foxit Software\Foxit Reader\LANGUAGE" `"langid"` `"40801d"`
			Goto LanguageEnd

	GetAppLanguage:		
		StrCmp $APPLANGUAGE "1025" 0 +3 			; Arabic
			StrCpy $APPLANGUAGE "40802d"
			Goto SetAppLanguage
		StrCmp $APPLANGUAGE "1251" 0 +3 			; Bulgarian
			StrCpy $APPLANGUAGE "40803d"
			Goto SetAppLanguage
		StrCmp $APPLANGUAGE "1027" 0 +3 			; Catalan
			StrCpy $APPLANGUAGE "40805d"
			Goto SetAppLanguage
		StrCmp $APPLANGUAGE "936" 0 +3 				; Chinese-Simplified
			StrCpy $APPLANGUAGE "40807d"
			Goto SetAppLanguage
		StrCmp $APPLANGUAGE "1028" 0 +3 			; Chinese-Traditional
			StrCpy $APPLANGUAGE "40808d"
			Goto SetAppLanguage
		StrCmp $APPLANGUAGE "1029" 0 +3 			; Czech
			StrCpy $APPLANGUAGE "40806d"
			Goto SetAppLanguage
		StrCmp $APPLANGUAGE "1030" 0 +3 			; Danish
			StrCpy $APPLANGUAGE "40809d"
			Goto SetAppLanguage
		StrCmp $APPLANGUAGE "1043" 0 +3 			; Dutch
			StrCpy $APPLANGUAGE "40810d"
			Goto SetAppLanguage
		StrCmp $APPLANGUAGE "1061" 0 +3 			; Estonian
			StrCpy $APPLANGUAGE "40812d"
			Goto SetAppLanguage
		StrCmp $APPLANGUAGE "1035" 0 +3 			; Finnish
			StrCpy $APPLANGUAGE "40813d"
			Goto SetAppLanguage
		StrCmp $APPLANGUAGE "1036" 0 +3 			; French
			StrCpy $APPLANGUAGE "40814d"
			Goto SetAppLanguage
		StrCmp $APPLANGUAGE "1031" 0 +3 			; German
			StrCpy $APPLANGUAGE "40816d"
			Goto SetAppLanguage
		StrCmp $APPLANGUAGE "1032" 0 +3 			; Greek
			StrCpy $APPLANGUAGE "40817d"
			Goto SetAppLanguage
		StrCmp $APPLANGUAGE "1037" 0 +3 			; Hebrew
			StrCpy $APPLANGUAGE "40832d"
			Goto SetAppLanguage
		StrCmp $APPLANGUAGE "2108" 0 +3 			; Irish
			StrCpy $APPLANGUAGE "40836d"
			Goto SetAppLanguage
		StrCmp $APPLANGUAGE "1040" 0 +3 			; Italian
			StrCpy $APPLANGUAGE "40818d"
			Goto SetAppLanguage
		StrCmp $APPLANGUAGE "1041" 0 +3 			; Japanese
			StrCpy $APPLANGUAGE "40833d"
			Goto SetAppLanguage
		StrCmp $APPLANGUAGE "1042" 0 +3 			; Korean
			StrCpy $APPLANGUAGE "40819d"
			Goto SetAppLanguage
		StrCmp $APPLANGUAGE "1044" 0 +3 			; Norwegian
			StrCpy $APPLANGUAGE "40822d"
			Goto SetAppLanguage
		StrCmp $APPLANGUAGE "1045" 0 +3 			; Polish
			StrCpy $APPLANGUAGE "40823d"
			Goto SetAppLanguage
		StrCmp $APPLANGUAGE "2070" 0 +3 			; Portuguese
			StrCpy $APPLANGUAGE "40824d"
			Goto SetAppLanguage
		StrCmp $APPLANGUAGE "1046" 0 +3 			; Portuguese Brazilian
			StrCpy $APPLANGUAGE "40825d"
			Goto SetAppLanguage
		StrCmp $APPLANGUAGE "1049" 0 +3 			; Russian
			StrCpy $APPLANGUAGE "40827d"
			Goto SetAppLanguage
		StrCmp $APPLANGUAGE "1060" 0 +3 			; Slovenian
			StrCpy $APPLANGUAGE "40828d"
			Goto SetAppLanguage
		StrCmp $APPLANGUAGE "1034" 0 +3 			; Spanish
			StrCpy $APPLANGUAGE "40829d"
			Goto SetAppLanguage
		StrCmp $APPLANGUAGE "1053" 0 +3 			; Swedish
			StrCpy $APPLANGUAGE "40830d"
			Goto SetAppLanguage
		StrCmp $APPLANGUAGE "1055" "" LanguageEnd	; Turkish
			StrCpy $APPLANGUAGE "40831d"

	SetAppLanguage:
		ReadINIStr $0 "$SETTINGSDIRECTORY\${NAME}.reg" "HKEY_CURRENT_USER\Software\Foxit Software\Foxit Reader\LANGUAGE" `"langid"`
		StrCmp $APPLANGUAGE $0 LanguageEnd ; if the same, move on
		WriteINIStr "$SETTINGSDIRECTORY\${NAME}.reg" "HKEY_CURRENT_USER\Software\Foxit Software\Foxit Reader\LANGUAGE" `"langid"` `"$APPLANGUAGE"`

	LanguageEnd:

	; MoveSettings:
		Rename "$APPDATA\Foxit Software\Foxit Reader" "$APPDATA\Foxit Software\Foxit Reader-BackupBy${NAME}"

	; RegistryBackup:
		${registry::KeyExists} "HKEY_CURRENT_USER\Software\Foxit Software\Foxit Reader-BackupBy${NAME}" $R0
		StrCmp $R0 "0" RestoreTheKey
		${registry::KeyExists} "HKEY_CURRENT_USER\Software\Foxit Software\Foxit Reader" $R0
		StrCmp $R0 "-1" RestoreTheKey
		${registry::MoveKey} "HKEY_CURRENT_USER\Software\Foxit Software\Foxit Reader" "HKEY_CURRENT_USER\Software\Foxit Software\Foxit Reader-BackupBy${NAME}" $R0
		Sleep 100

	RestoreTheKey:
		IfFileExists "$SETTINGSDIRECTORY\${NAME}.reg" "" BackupHKLM

		IfFileExists "$WINDIR\system32\reg.exe" "" RestoreTheKey9x
			nsExec::ExecToStack `"$WINDIR\system32\reg.exe" import "$SETTINGSDIRECTORY\${NAME}.reg"`
			Pop $R0
			StrCmp $R0 '0' BackupHKLM ; successfully restored key

	RestoreTheKey9x:
		${registry::RestoreKey} "$SETTINGSDIRECTORY\${NAME}.reg" $R0
		StrCmp $R0 '0' BackupHKLM ; successfully restored key
		StrCpy $FAILEDTORESTOREKEY "true"

	BackupHKLM:
		UserInfo::GetAccountType
		Pop $USERTYPE
		StrCmp $USERTYPE "Guest" LaunchNow
		StrCmp $USERTYPE "User" LaunchNow
		${registry::MoveKey} "HKEY_LOCAL_MACHINE\SOFTWARE\Classes\FoxitReader.Document" "HKEY_LOCAL_MACHINE\SOFTWARE\Classes\FoxitReader.Document-BackupBy${NAME}" $R0
		${registry::MoveKey} "HKEY_LOCAL_MACHINE\SOFTWARE\Classes\FoxitReader.FDFDoc" "HKEY_LOCAL_MACHINE\SOFTWARE\Classes\FoxitReader.FDFDoc-BackupBy${NAME}" $R0
		${registry::MoveKey} "HKEY_LOCAL_MACHINE\SOFTWARE\Classes\CLSID\{14E8BBD8-1D1C-4D56-A4DA-D20B75EB814E}" "HKEY_LOCAL_MACHINE\SOFTWARE\Classes\CLSID\{14E8BBD8-1D1C-4D56-A4DA-D20B75EB814E}-BackupBy${NAME}" $R0
		${registry::MoveKey} "HKEY_LOCAL_MACHINE\SOFTWARE\Classes\CLSID\{A43F4073-BEBD-4142-B590-27A74384620B}" "HKEY_LOCAL_MACHINE\SOFTWARE\Classes\CLSID\{A43F4073-BEBD-4142-B590-27A74384620B}-BackupBy${NAME}" $R0
		${registry::KeyExists} "HKEY_LOCAL_MACHINE\SOFTWARE\Classes\.pdf" $R0
		${If} $R0 == "0"
			StrCpy $RegKeyPDFExists true
			${registry::Read} "HKEY_LOCAL_MACHINE\SOFTWARE\Classes\.pdf" "" $R0 $R1
			${If} $R0 == ""
				StrCpy $RegKeyPDFIsBlank true
			${EndIf}
		${EndIf}
		${registry::KeyExists} "HKEY_LOCAL_MACHINE\SOFTWARE\Classes\.fdf" $R0
		${If} $R0 == "0"
			StrCpy $RegKeyFDFExists true
		${EndIf}
		
	LaunchNow:
		Sleep 100
		ExecWait $EXECSTRING

	CheckRunning:
		Sleep 1000
		FindProcDLL::FindProc "$PROGRAMEXECUTABLE"
		StrCmp $R0 "1" CheckRunning

	; MoveSettingsBack:
		Sleep 500
		RMDir /r "$APPDATA\Foxit Software\Foxit Reader"
		Rename "$APPDATA\Foxit Software\Foxit Reader-BackupBy${NAME}" "$APPDATA\Foxit Software\Foxit Reader"
		RMDir "$APPDATA\Foxit Software" ; if empty

	; RegistrySaveKey:
		StrCmp $FAILEDTORESTOREKEY "true" SetOriginalKeyBack
		${registry::SaveKey} "HKEY_CURRENT_USER\Software\Foxit Software\Foxit Reader" "$SETTINGSDIRECTORY\${NAME}.reg" "" $0
		Sleep 100

	SetOriginalKeyBack:
		${registry::DeleteKey} "HKEY_CURRENT_USER\Software\Foxit Software\Foxit Reader" $R0
		Sleep 100
		${registry::KeyExists} "HKEY_CURRENT_USER\Software\Foxit Software\Foxit Reader-BackupBy${NAME}" $R0
		StrCmp $R0 "-1" CleanupHKLM
		${registry::MoveKey} "HKEY_CURRENT_USER\Software\Foxit Software\Foxit Reader-BackupBy${NAME}" "HKEY_CURRENT_USER\Software\Foxit Software\Foxit Reader" $R0
		Sleep 100

	CleanupHKLM:
		StrCmp $USERTYPE "Guest" RegistryCleanup
		StrCmp $USERTYPE "User" RegistryCleanup
		${registry::DeleteKey} "HKEY_LOCAL_MACHINE\SOFTWARE\Classes\FoxitReader.Document" $R0
		${registry::MoveKey} "HKEY_LOCAL_MACHINE\SOFTWARE\Classes\FoxitReader.Document-BackupBy${NAME}" "HKEY_LOCAL_MACHINE\SOFTWARE\Classes\FoxitReader.Document" $R0
		${registry::DeleteKey} "HKEY_LOCAL_MACHINE\SOFTWARE\Classes\FoxitReader.FDFDoc" $R0
		${registry::MoveKey} "HKEY_LOCAL_MACHINE\SOFTWARE\Classes\FoxitReader.FDFDoc-BackupBy${NAME}" "HKEY_LOCAL_MACHINE\SOFTWARE\Classes\FoxitReader.FDFDoc" $R0
		${registry::DeleteKey} "HKEY_LOCAL_MACHINE\SOFTWARE\Classes\CLSID\{14E8BBD8-1D1C-4D56-A4DA-D20B75EB814E}" $R0
		${registry::MoveKey} "HKEY_LOCAL_MACHINE\SOFTWARE\Classes\CLSID\{14E8BBD8-1D1C-4D56-A4DA-D20B75EB814E}-BackupBy${NAME}" "HKEY_LOCAL_MACHINE\SOFTWARE\Classes\CLSID\{14E8BBD8-1D1C-4D56-A4DA-D20B75EB814E}" $R0
		${registry::DeleteKey} "HKEY_LOCAL_MACHINE\SOFTWARE\Classes\CLSID\{A43F4073-BEBD-4142-B590-27A74384620B}" $R0
		${registry::MoveKey} "HKEY_LOCAL_MACHINE\SOFTWARE\Classes\CLSID\{A43F4073-BEBD-4142-B590-27A74384620B}-BackupBy${NAME}" "HKEY_LOCAL_MACHINE\SOFTWARE\Classes\CLSID\{A43F4073-BEBD-4142-B590-27A74384620B}" $R0
		${If} $RegKeyPDFExists != true
			${registry::DeleteKey} "HKEY_LOCAL_MACHINE\SOFTWARE\Classes\.pdf" $R0
		${EndIf}
		${If} $RegKeyPDFIsBlank == true
			${registry::Read} "HKEY_LOCAL_MACHINE\SOFTWARE\Classes\.pdf" "" $R0 $R1
			${If} $R0 != ""
				${registry::DeleteValue} "HKEY_LOCAL_MACHINE\SOFTWARE\Classes\.pdf" "" $R0
			${EndIf}
		${EndIf}
		${If} $RegKeyFDFExists != true
			${registry::DeleteKey} "HKEY_LOCAL_MACHINE\SOFTWARE\Classes\.fdf" $R0
		${EndIf}

	RegistryCleanup:
		${registry::DeleteKeyEmpty} "HKEY_CURRENT_USER\Software\Foxit Software" $R0
		Goto TheEnd

	LaunchAndExit:
		Exec $EXECSTRING

	TheEnd:
		${registry::Unload}
		newadvsplash::stop /WAIT
SectionEnd